class Town {
	id = 0;
	constructor (t_id) {
		this.id = t_id;
	}
}

function Town::ManageTown() {
	local townName = GSTown.GetName(this.id);
	local growthRateMultiplier = 0;
		local GRSetting = 0;
	if (GSGameSettings.IsValid("town_growth_rate")) {
		GRSetting = GSGameSettings.GetValue("town_growth_rate");
		if (GRSetting) {
			growthRateMultiplier = 1.0 / GRSetting;
		}
	}
	
	if (growthRateMultiplier == 0) {
		Log.Info("Growth rate for " + townName + " was not set.", Log.LVL_DEBUG);
		return(false);  // if growth rate is set to "none", then leave the growth rate alone.
	}
	
	local paxTransport = GSTown.GetLastMonthTransportedPercentage (this.id, Helper.GetPAXCargo());
	local mailTransport = GSTown.GetLastMonthTransportedPercentage (this.id, Helper.GetMailCargo());
	if (paxTransport < GSController.GetSetting("min_transport_pax") || mailTransport < GSController.GetSetting("min_transport_mail")) { 
		GSTown.SetGrowthRate(this.id, 365*1500);
		Log.Info("Growth rate for " + townName + " was set to 1500 years.", Log.LVL_DEBUG);
		if (paxTransport < GSController.GetSetting("min_transport_pax")) {
			Log.Info("--" + paxTransport + "% of PAX transported is below threashold of " + GSController.GetSetting("min_transport_pax"), Log.LVL_SUB_DECISIONS);
		}			
		if (mailTransport < GSController.GetSetting("min_transport_mail")) {
			Log.Info("--" + mailTransport + "% of Mail transported is below threashold of " + GSController.GetSetting("min_transport_mail"), Log.LVL_SUB_DECISIONS);
		}
		return(false);			
	}
	
	local perfectRate = 100;
	local paxMultiplier = GSTown.IsCity(this.id)?3:6;
	local mailMultiplier = GSTown.IsCity(this.id)?1:2;
		
	local growthRate = 0;
	local paxRate = 0;
	local mailRate = 0;
	local sizeBasedRate = 0;

	try {
		paxRate = max((perfectRate - paxTransport),0) * paxMultiplier;
		mailRate = max((perfectRate - mailTransport),0) * mailMultiplier;
		sizeBasedRate = max((GSTown.GetPopulation(this.id) / 75).tointeger(),1); 
		growthRate =  ((paxRate + mailRate + sizeBasedRate ) * growthRateMultiplier).tointeger();
	} catch (error) {
		growthRate = 365 * 1000;
	}

	if (growthRate < 1) { growthRate = 1; }

	local success = GSTown.SetGrowthRate(this.id, growthRate);
	Log.Info("Growth rate for " + townName + " was set to " + growthRate + ".", Log.LVL_DEBUG);
	Log.Info("--Mail percent: " + mailTransport + "; Pax percent: " + paxTransport, Log.LVL_DEBUG);
	Log.Info("--Mail rate: " + mailRate + "; Pax rate: " + paxRate + "; Size rate: " + sizeBasedRate, Log.LVL_DEBUG);
	return(true);
}